#include "Enemy.h"
#include "debug.h"
#include "Messages.h"

Enemy* MakeEnemy() {
    Enemy* self = malloc(sizeof(Enemy));
    ASSERT_RETURN(self != NULL, NULL, "Failed to allocate Enemy");
    *self = (Enemy){
        .transform = IdentityTransform,
        .behaviour = NULL,
        .rigidbody = NULL,
        .collider = NULL,
        .sprite = sprite_new_empty(),
        .health = 5,
    };
    self->rigidbody = rigidbody_make(Enemy_as_PhysicsEntity(self));
    return self;
}

Enemy* SpawnEnemy(Vector location, const State* entryState) {
    Enemy* self = MakeEnemy();
    self->behaviour = state_machine_init(self, entryState);
    return self;
}

void EnemyStart(Enemy* self) {}

void EnemyUpdate(Enemy* self, float deltaTime) {
    state_machine_update(self->behaviour, deltaTime);
}

void EnemyDraw(Enemy* self) {}

void EnemyDestroy(Enemy* self) {
    state_machine_destroy(self->behaviour);
    collider_destroy(self->collider);
    rigidbody_destroy(self->rigidbody);
    sprite_destroy(self->sprite);
    free(self);
}

void EnemyOnCollision(Enemy* self, Collision collision) {}
void EnemyOnOverlap(Enemy* self, Collider* other) {}

void EnemyHandleMessage(Enemy* self, MessageID id, void* data) {
    switch(id) {
    default: return;
    case MESSAGE_DEAL_DAMAGE:
        self->health -= (uintptr_t)data;
        break;
    }
}

Transform* EnemyGetTransform(Enemy* self) {
    return &self->transform;
}

RigidBody* EnemyGetRigidBody(Enemy* self) {
    return self->rigidbody;
}
